#include "SubjectIf.h"
#include "ObserverIf.h"

#include <iostream>
#include <mutex>
#include <set>

namespace
{


class Subject : public SubjectIf
{
public:
    Subject() :
        mObservers{},
        mMutex{}
    {
    }

    void registerObserver(ObserverIf& observer) override
    {
        std::lock_guard lock{mMutex};
        mObservers.insert(&observer);
    }

    void unRegisterObserver(ObserverIf& observer) override
    {
        std::lock_guard lock{mMutex};
        mObservers.erase(&observer);
    }

    void actAsIfStateWouldChange()
    {
        std::lock_guard lock{mMutex};
        notify(lock);
    }

private:
    void notify(const std::lock_guard<std::mutex>&)  /// :-(    ......
    {
        for(auto observer : mObservers)
        {
            observer->update();
        }
    }

    std::set<ObserverIf*> mObservers;
    std::mutex mMutex;
};



class Observer : public ObserverIf
{
public:
    explicit Observer(const std::string& name) :
        mName{name}
    {
    }

    void update() override
    {
        std::cout << mName << std::endl;
    }
private:
    std::string mName;
};

} // namespace

int main()
{
    Subject sub;

    Observer foo{"foo"};
    Observer bar{"bar"};
    Observer real{"real"};
    Observer nope{"nope"};

    sub.registerObserver(foo);
    sub.registerObserver(bar);
    sub.registerObserver(real);
    sub.registerObserver(nope);

    sub.unRegisterObserver(nope);

    sub.actAsIfStateWouldChange();


    /// unregister time
    sub.unRegisterObserver(foo);
    sub.unRegisterObserver(bar);
    sub.unRegisterObserver(real);
}
