#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY

#include <iostream>
#include <thread>


/// OBSERVER PATTERN
///
///     => subject      ---> signal
///
///     => observer     ---> slot
///
///     => notification (signal) going from subject to observer(s)

/// observer registers on the subject
///
///     => observer CONNECTS on the subject ===> CONNECTION is created
///
///     NOT !!!!!!!  RAII : keep track of the connection in order to unregister
///
///     => registers the "CALLBACK", the entity to call (aka signal)


namespace
{

class Subject
{
public:
    boost::signals2::connection registerObserver(
            const std::function<void ()>& handler)
    {
        return mSignalHandlers.connect(handler);
    }

    void notify()
    {
        mSignalHandlers();
    }

private:

    boost::signals2::signal<void ()> mSignalHandlers; /// aka the observers
};

void foo()
{
    std::cout << "foo" << std::endl;
}

void bar()
{
    std::cout << "bar" << std::endl;
}

void real()
{
    std::cout << "real" << std::endl;
}

void nope()
{
    std::cout << "nope" << std::endl;
}

} // namespace

int main()
{
    Subject sub;

    sub.registerObserver(foo);

    sub.registerObserver(bar);

    auto connectionReal = sub.registerObserver(real);

    auto connectionNope = sub.registerObserver(nope);

    connectionNope.disconnect();

    sub.notify();
}
