#include <boost/signals2.hpp>

#include <iostream>
#include <thread>


/// OBSERVER PATTERN
///
///     => subject      ---> signal
///
///     => observer     ---> slot
///
///     => notification (signal) going from subject to observer(s)

/// observer registers on the subject
///
///     => observer CONNECTS on the subject ===> CONNECTION is created
///
///     NOT !!!!!!!  RAII : keep track of the connection in order to unregister
///
///     => registers the "CALLBACK", the entity to call (aka signal)

/// make the connection RAII ===> boost::signals2::scoped_connection


namespace
{

class Subject
{
public:
    boost::signals2::scoped_connection registerObserver(
            const std::function<void ()>& handler)
    {
        return mSignalHandlers.connect(handler);
    }

    void notify()
    {
        mSignalHandlers();
    }

private:

    boost::signals2::signal<void ()> mSignalHandlers; /// aka the observers
};

void foo()
{
    std::cout << "foo" << std::endl;
}

void bar()
{
    std::cout << "bar" << std::endl;
}

void real()
{
    std::cout << "real" << std::endl;
}

void nope()
{
    std::cout << "nope" << std::endl;
}

} // namespace

int main()
{
    Subject sub;

    {
        auto connectionFoo{sub.registerObserver(foo)};
        auto connectionBar{sub.registerObserver(bar)};
        auto connectionReal{sub.registerObserver(real)};
        auto connectionNope{sub.registerObserver(nope)};
        auto connectionLambda{sub.registerObserver([](){std::cout << "lambda" << std::endl;})};

        connectionNope.disconnect();

        sub.notify();
    }

    std::cout << "once more " << std::endl;
    sub.notify();
}
