#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY

#include <iostream>
#include <functional>

/// block the connection for a short moment
/// without removing it definitely, we keep the association/registration


namespace
{

void foo()
{
    std::cout << "foo" << std::endl;
}

void bar()
{
    std::cout << "bar" << std::endl;
}

} // namespace

int main()
{
    boost::signals2::signal<void ()> sig;

    auto conFoo = sig.connect(foo);
    auto conBar = sig.connect(bar);
    auto conLambda = sig.connect([](){std::cout << "lambda" << std::endl;});

    sig();

    std::cout << "\n\n";
    /// let's block a few things

    boost::signals2::shared_connection_block blockFoo{conFoo};

    {
        boost::signals2::shared_connection_block blockLambda{conLambda};
        sig();
        std::cout << "\n\n";
    }

    sig();
    std::cout << "\n\n";

    /// we can play with the block
    std::cout << "Foo blocking : " << std::boolalpha << blockFoo.blocking() << std::endl;

    blockFoo.unblock();
    std::cout << "Foo blocking : " << std::boolalpha << blockFoo.blocking() << std::endl;
    sig();
    std::cout << "\n\n";

    blockFoo.block();
    std::cout << "Foo blocking : " << std::boolalpha << blockFoo.blocking() << std::endl;
    sig();
    std::cout << "\n\n";
}
