#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY

#include <iostream>
#include <functional>

/// when a slot gets connected to a signal ==> return a connection object

namespace
{

void foo()
{
    std::cout << "foo" << std::endl;
}

void bar()
{
    std::cout << "bar" << std::endl;
}

class Test
{
public:
    Test() = default;

    void operator()()
    {
        std::cout << "Test" << std::endl;
    }
};

class Yoyo
{
public:
    Yoyo() = default;

    void doSomething()
    {
        std::cout << "Yoyo" << std::endl;
    }
};

void manual()
{
    boost::signals2::signal<void ()> sig;

    boost::signals2::connection conFoo = sig.connect(foo);
    auto conBar = sig.connect(bar);
    auto conLambda = sig.connect([](){std::cout << "lambda" << std::endl;});
    Test test;
    auto conTest = sig.connect(test);

    Yoyo yoyo;
    auto conYoyo = sig.connect(std::bind(&Yoyo::doSomething, yoyo));

    sig();

    std::cout << "\n\n";
    /// let's disconnect a few things
    std::cout << "foo still connected : " << std::boolalpha << conFoo.connected() << std::endl;
    conFoo.disconnect();
    std::cout << "foo still connected : " << std::boolalpha << conFoo.connected() << std::endl;
    conLambda.disconnect();
    conTest.disconnect();
    conYoyo.disconnect();

    sig();

    std::cout << "\n\n";
}

void goingOutOfScope()
{
    boost::signals2::signal<void ()> sig;
    {
        boost::signals2::scoped_connection conFoo = sig.connect(foo); /// scoped_connection
        boost::signals2::scoped_connection conBar = sig.connect(bar);

        sig();
        std::cout << "\n\n";
    }
    sig();
}

} // namespace

int main()
{
    manual();
    goingOutOfScope();
}
