#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY

#include <iostream>
#include <functional>

/// events, observation ==> more general : SIGNAL ==> SLOT
///
///     a signal send information to one or more interested slots
///         ==> there is as such a connection between the signal and the slot

///         ===> connection can be broken by "disconnect"

/// the signal expect slots of a given signature, this signature is a template type parameter

/// triggering the signal to notify (to signal) are the slots ==> function call () on the signal

namespace
{

void foo()
{
    std::cout << "foo" << std::endl;
}

void bar()
{
    std::cout << "bar" << std::endl;
}

class Test
{
public:
    Test() = default;

    void operator()()
    {
        std::cout << "Test" << std::endl;
    }
};

class Yoyo
{
public:
    Yoyo() = default;

    void doSomething()
    {
        std::cout << "Yoyo" << std::endl;
    }
};

} // namespace

int main()
{
    boost::signals2::signal<void ()> sig;

    sig.connect(foo);
    sig.connect(bar);
    sig.connect([](){std::cout << "lambda" << std::endl;});
    Test test;
    sig.connect(test);

    Yoyo yoyo;
    sig.connect(std::bind(&Yoyo::doSomething, yoyo));

    sig();

    std::cout << "\n\n";
    /// let's disconnect a few things
    sig.disconnect(foo);
//    sig.disconnect(test);
//    sig.disconnect(std::bind(&Yoyo::doSomething, yoyo));
    sig.disconnect([](){std::cout << "lambda" << std::endl;}); /// not what we expect

    sig();

    std::cout << "\n\n";
    /// brute force
    sig.disconnect_all_slots();
    sig();
}
