#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY
#include <boost/shared_ptr.hpp>

#include <iostream>
#include <functional>
#include <memory>

namespace
{

class Yoyo
{
public:
    Yoyo() = default;
    ~Yoyo() {std::cout << "die" << std::endl;}

    void doSomething()
    {
        std::cout << "Yoyo" << std::endl;
    }
};

void problematic()
{
    boost::signals2::signal<void ()> sig;

    {
        auto yoyo{std::make_unique<Yoyo>()};
        sig.connect(std::bind(&Yoyo::doSomething, yoyo.get()));
    }
    std::cout << sig.num_slots() << std::endl;
    sig();  /// all bets are off
}

void save1()
{
    boost::signals2::signal<void ()> sig;
    {
        boost::shared_ptr<Yoyo> yoyo{new Yoyo()};
        sig.connect(boost::signals2::signal<void ()>::slot_type(&Yoyo::doSomething, yoyo.get()).track(yoyo));
    }
    std::cout << sig.num_slots() << std::endl;
    sig();
}

void save2()
{
    boost::signals2::signal<void ()> sig;
    {
        auto yoyo{std::make_shared<Yoyo>()};
        sig.connect(boost::signals2::signal<void ()>::slot_type(&Yoyo::doSomething, yoyo.get()).track_foreign(yoyo));
    }
    std::cout << sig.num_slots() << std::endl;
    sig();
}

void save3()
{
    boost::signals2::signal<void ()> sig;
    {
        auto yoyo{std::make_shared<Yoyo>()};
        std::weak_ptr<Yoyo> weakYoyo{yoyo};
        sig.connect(boost::signals2::signal<void ()>::slot_type(&Yoyo::doSomething, yoyo.get()).track_foreign(weakYoyo));
    }
    std::cout << sig.num_slots() << std::endl;
    sig();
}

} // namespace

int main()
{
    problematic();

    std::cout << std::endl;

    save1();

    std::cout << std::endl;

    save2();

    std::cout << std::endl;

    save3();
}
