#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY

#include <iostream>
#include <functional>

/// say, we have a signal that returns a value

namespace
{

int foo()
{
    std::cout << "foo" << std::endl;
    return 101;
}

int bar()
{
    std::cout << "bar" << std::endl;
    return 242;
}

class Test
{
public:
    Test() = default;

    int operator()()
    {
        std::cout << "Test" << std::endl;
        return 42;
    }
};

class Yoyo
{
public:
    Yoyo() = default;

    int doSomething()
    {
        std::cout << "Yoyo" << std::endl;
        return 911;
    }
};

} // namespace

int main()
{
    boost::signals2::signal<int ()> sig;

    sig.connect(foo);
    sig.connect(bar);
    sig.connect([](){std::cout << "lambda" << std::endl; return 55;});
    Test test;
    sig.connect(test);

    Yoyo yoyo;
    sig.connect(std::bind(&Yoyo::doSomething, yoyo));

    /// last value
    std::cout << *sig();   /// dereferencing a boost::optional

    std::cout << std::endl << std::endl;

    sig.disconnect_all_slots();
    const auto retOptional = sig();
    if(!retOptional)
    {
        std::cout << "no value" << std::endl;
    }
}
