#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY

#include <iostream>
#include <functional>
#include <vector>
#include <algorithm>

/// say, we have a signal that returns a value

/// COMBINE them ==> extra template argument : combiner
/// combiner : operator() ==> 2 iterators

namespace
{

int foo()
{
    std::cout << "foo" << std::endl;
    return 101;
}

int bar()
{
    std::cout << "bar" << std::endl;
    return 242;
}

class Test
{
public:
    Test() = default;

    int operator()()
    {
        std::cout << "Test" << std::endl;
        return 42;
    }
};

class Yoyo
{
public:
    Yoyo() = default;

    int doSomething()
    {
        std::cout << "Yoyo" << std::endl;
        return 911;
    }
};

struct minimumElement
{
    using result_type = int; /// this is required ==> as such a default std algorithm can not be used directly

    template<typename InputIterator>
    result_type operator()(InputIterator first, InputIterator last) const
    {
        std::vector<result_type> vec(first, last); /// here the iterators get processed and the slots get called
        return *std::min_element(vec.cbegin(), vec.cend());
    }
};

} // namespace

int main()
{
    boost::signals2::signal<int (), minimumElement> sig;

    sig.connect(foo);
    sig.connect(bar);
    sig.connect([](){std::cout << "lambda" << std::endl; return 55;});
    Test test;
    sig.connect(test);

    Yoyo yoyo;
    sig.connect(std::bind(&Yoyo::doSomething, yoyo));

    std::cout << sig();
}
