#include <boost/signals2.hpp>   /// HEADER ONLY LIBRARY

#include <iostream>
#include <functional>
#include <optional>

/// say, we have a signal that returns a value

/// COMBINE them ==> extra template argument : combiner
/// combiner : operator() ==> 2 iterators

namespace
{

int foo()
{
    std::cout << "foo" << std::endl;
    return 101;
}

int bar()
{
    std::cout << "bar" << std::endl;
    return 242;
}

class Test
{
public:
    Test() = default;

    int operator()()
    {
        std::cout << "Test" << std::endl;
        return 42;
    }
};

class Yoyo
{
public:
    Yoyo() = default;

    int doSomething()
    {
        std::cout << "Yoyo" << std::endl;
        return 911;
    }
};

struct minimumElement
{
    using result_type = std::optional<int>;

    template<typename InputIterator>
    result_type operator()(InputIterator first, InputIterator last) const
    {
        std::optional<int> result;
        while(first != last)
        {
            result = *first;
            ++first;
        }
        return result;
    }
};

} // namespace

int main()
{
    boost::signals2::signal<int (), minimumElement> sig;

    sig.connect(foo);
    sig.connect(bar);
    sig.connect([](){std::cout << "lambda" << std::endl; return 55;});
    Test test;
    sig.connect(test);
    Yoyo yoyo;
    sig.connect(std::bind(&Yoyo::doSomething, yoyo));

    /// last value
    std::cout << *sig();   /// dereferencing a std::optional

    std::cout << std::endl << std::endl;

    sig.disconnect_all_slots();
    const auto retOptional = sig();
    if(!retOptional)
    {
        std::cout << "no value" << std::endl;
    }
}
